from .team import Team

class Match:

    def __init__(self, home_team, away_team, home_score=0, away_score=0, date="00-00-0000"):
        """
        Def:
        This is the function of initialization of a match.

        Args:
            home_team ([object Team]): The home team of the match.
            away_team ([object Team]): The away team of the match.
            home_score (int, optional): The score of the home team. Defaults to 0.
            away_score (int, optional): The score of the away team. Defaults to 0.
            date (str, optional): The date of the match. Defaults to "00-00-0000".
        """
        self.home_team = home_team
        self.away_team = away_team
        self.home_score = home_score
        self.away_score = away_score
        self.date = date

        self.goals_total = home_score + away_score
        self.goals_difference = abs(home_score - away_score)

        self.winner = self.define_winner()

    def __repr__(self):
        """
        Def:
        This function rewrite the representation default of the Python, so this function will be executed
        when the object will printed.
        """
        text = "{} ({}) x ({}) {} ({}) \n\n".format(self.home_team.name, self.home_score, self.away_score, self.away_team.name, self.date)

        if(self.winner != None):
            text += "The winner was {} with {} goals of difference.".format(self.winner.name, self.goals_difference)
        else:
            text += "The match ended drew."

        return text

    def define_winner(self):
        """
        Def:
        This function return which team won the match

        Returns:
            [object Team]: The winner team, if the match ended drew the value returned is None
        """
        if ( self.home_score > self.away_score ):
            return self.home_team
        elif ( self.home_score == self.away_score ):
            return None
        else:
            return self.away_team


import pandas as pd

class FootpyManager:

    def __init__(self):
        self.teams = pd.DataFrame(columns=["id", "name", "country"])
        self.matches = pd.DataFrame(
            columns=["home_id", "home_name", "home_score",
                     "away_id", "away_name", "away_score",
                     "goals_total", "goals_difference", "winner",
                     "date"]
            )

    def insert_match(self, match):
        """
        Def:
        This function insert a match in the manager and insert the teams in the dataframe of teams.

        Args:
            match ([object Match]): The register of the match
        """
        has_home_team = False
        has_away_team = False

        for index, team in self.teams.iterrows():
            if team.id == match.home_team.id: has_home_team = True
            if team.id == match.away_team.id: has_away_team = True

        if (has_home_team == False): self.insert_team(match.home_team)
        if (has_away_team == False): self.insert_team(match.away_team)

        self.matches = self.matches.append({
            'home_id': match.home_team.id,
            'home_name': match.home_team.name,
            'home_score': match.home_score,
            'away_id': match.away_team.id,
            'away_name': match.away_team.name,
            'away_score': match.away_score,
            'goals_total': match.goals_total,
            'goals_difference': match.goals_difference,
            'winner': match.winner.name if (match.winner != None) else None,
            'date': match.date,
            'object': match
        }, ignore_index=True)

    def insert_team(self, team):
        """
        Def:
        This function insert a team in the dataframe.

        Args:
            team ([object Team]): The register of the team
        """
        self.teams = self.teams.append({
            'id': team.id,
            'name': team.name,
            'country': team.country
        }, ignore_index=True)

    def match_with_more_goals(self, team=None):
        """
        Def:
        This function return the match with the most goals in the total. If a team is inputted
        the function filter the matches of this team.

        Args:
            team ([object Team], optional): Filters the matches of a team. Defaults to None.

        Returns:
            [object Match]: The match with the most goals in total
        """
        matches = self.matches
        if(team != None):
            matches = self.matches.query("home_id == '"+team.id+"' | away_id == '"+team.id+"'")

        return matches.sort_values(by='goals_total', ascending=False).head(1).object.item()

    def match_with_most_difference_goals(self, team=None):
        """
        Def:
        This function return the match with the most difference of goals. If a team is inputted
        the function filter the matches of this team.

        Args:
            team ([object Team], optional): Filters the matches of a team. Defaults to None.

        Returns:
            [object Match]: The match with the most difference of goals
        """
        matches = self.matches
        if(team != None):
            matches = self.matches.query("home_id == '"+team.id+"' | away_id == '"+team.id+"'")

        return matches.sort_values(by='goals_difference', ascending=False).head(1).object.item()

    def mean_goals_made(self, team):
        """
        Def:
        This function return the mean of goals made by the team inputted.

        Args:
            team ([object Team]): The team to be analyzed.

        Returns:
            [float]: The mean of goals made by the team selected.
        """
        matches_filtered = self.matches.query("home_id == '"+team.id+"' | away_id == '"+team.id+"'")
        home_score_mean = matches_filtered[matches_filtered['home_id'] == team.id]['home_score'].sum()
        away_score_mean = matches_filtered[matches_filtered['away_id'] == team.id]['away_score'].sum()

        return (home_score_mean + away_score_mean) / matches_filtered.shape[0]

    def mean_goals_suffered(self, team):
        """
        Def:
        This function return the mean of goals suffered by the team inputted.

        Args:
            team ([object Team]): The team to be analyzed.

        Returns:
            [float]: The mean of goals suffered by the team selected.
        """
        matches_filtered = self.matches.query("home_id == '"+team.id+"' | away_id == '"+team.id+"'")
        home_score_mean = matches_filtered[matches_filtered['home_id'] == team.id]['away_score'].sum()
        away_score_mean = matches_filtered[matches_filtered['away_id'] == team.id]['home_score'].sum()

        return (home_score_mean + away_score_mean) / matches_filtered.shape[0]



from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='footpy_manager',
      version='0.3',
      description='A Python package to manage matches of football.',
      packages=['footpy_manager'],
      long_description=long_description,
      long_description_content_type='text/markdown',
      zip_safe=False)


class Team:

    def __init__(self, name, country=""):
        """
        Def:
        This function is executed when a team is initialized. The ID was created to identify duplicate teams.

        Args:
            name (str): The name of the team.
            country (str, optional): The country of the team. Defaults to "".
        """
        self.name = name
        self.country = country
        self.id = str(name).lower().strip()

    def set_country(self, country):
        '''
        This function able to change the country of the team.
        '''
        self.country = country